#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::ProductUploadService
# state        public
#----------------------------------------------------------------------------------------
# description  GoogleSmartShoppingScriptTagService class
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::ProductUploadService;

use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::GoogleSmartShopping::API::Constants qw ( NUMBER_PRODUCTS_REQUEST_THRESHOLD );

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::GoogleSmartShopping::API::ProductUploadService->new( $Shop );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::GoogleSmartShopping::API::ProductUploadService
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop)  = @_;

    my $self = bless {
        'Shop' => $Shop,
        'Products' => [],
    }, $class;

    return $self;
}

#========================================================================================
# function     ExportItem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductUploadService->ExportItem( $hProduct );
#----------------------------------------------------------------------------------------
# description  Receives each product data and saves it on $self->{'Products'}
#----------------------------------------------------------------------------------------
# input        $hProduct | a product data for the export | refhash
#========================================================================================
sub ExportItem {
    my $self = shift;
    my ($hProduct)  = @_;

    push(@{$self->{'Products'}}, {
        'method' => 'insert',
        'product' => $hProduct,
    });

    if (scalar @{$self->{'Products'}} >= NUMBER_PRODUCTS_REQUEST_THRESHOLD) {
        $self->_sendInsertBatchRequest();
    }

    return;
}

#========================================================================================
# function     OnStartExport
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductUploadService->OnStartExport();
#----------------------------------------------------------------------------------------
# description  OnStartExport gets called before the start of the iteration of products
#               It loads the GoogleApiService necessary to make the insert requests
#========================================================================================
sub OnStartExport {
    my $self = shift;
 
    $self->{'GoogleApiService'} = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $self->{'Shop'}});
    $self->{'GoogleApiService'}->loadGoogleSettings();

    $self->{'MerchantCenterAccountID'} = $self->{'GoogleApiService'}->{'GoogleSmartShoppingConfig'}->get('MerchantCenterAccountID');

    return;
}

#========================================================================================
# function     OnEndExport
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductUploadService->OnEndExport();
#----------------------------------------------------------------------------------------
# description  OnEndExport gets called at the end of the iteration of products
#               It triggers _sendInsertBatchRequest if products are waiting in queue
#========================================================================================
sub OnEndExport {
    my $self = shift;
 
    if (scalar @{$self->{'Products'}}) {
        $self->_sendInsertBatchRequest();
    }

    return;
}

sub _sendInsertBatchRequest {
    my $self = shift;

    my $hProductBatchInsertPayload = {
        'entries' => $self->{'Products'}
    };

    my $hResponse;
    eval {
        $hResponse = $self->{'GoogleApiService'}->callProductsBatch($hProductBatchInsertPayload);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('sendInsertBatchRequest failed', {
            'Error' => $Error->message(),
            'Response' => $hResponse,
        });
    }

    $self->{'Products'} = [];

    return;
}

1;
