#==============================================================================
# §package      DE_EPAGES::GoogleSmartShopping::UI::Campaigns
# §state        private
#==============================================================================
package DE_EPAGES::GoogleSmartShopping::UI::Campaigns;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::Core::API::PerlTools qw ( isValid GetCurrentDateTime );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService;
use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;
use DE_EPAGES::GoogleSmartShopping::API::GooglePortalService;
use DE_EPAGES::GoogleSmartShopping::API::ScriptTagService;
use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping GetErrorCode );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    ERROR_FORM_SUBMIT
    ERROR_ACCEPT_TERMS
    ERROR_VALIDATION
);
use HTTP::Status qw ( HTTP_NOT_FOUND HTTP_OK );

use DateTime;

use strict;

#==============================================================================
# Event handlers
#==============================================================================
sub UnityViewGoogleSmartShoppingCampaigns {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    my $GoogleSmartShoppingPortalConfig = $self->_getPortalConfig($Shop);
    if (defined($GoogleSmartShoppingPortalConfig)) {
        $Servlet->vars('PortalConfigID', $GoogleSmartShoppingPortalConfig->id());
    }

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    my $GoogleAdsCampaign = $self->GetGoogleAdsCampaign($Shop, $GoogleSmartShoppingConfig);

    my $hRatedProductsStatistics = $self->_getRatedProductStatistics($Shop);
    $Servlet->vars('RatedProductsStatistics', $hRatedProductsStatistics);

    $Servlet->vars('CampaignCreationAllowed', 0);
    if (defined $GoogleAdsCampaign) {
        $Servlet->vars('GoogleAdsCampaign', $GoogleAdsCampaign);
        if (!defined($GoogleAdsCampaign->{'name'})) {
            $Servlet->vars('ErrorGettingExistingCampaign', 1);
        }
        return $self->SUPER::View($Servlet);
    }

    unless ($self->_areCampaignCreationRequirementsMet($Shop)) {
        $Servlet->vars('ForbidCampaignCreation', { IsOnboardingComplete => 0 });
        return $self->SUPER::View($Servlet);
    }

   if (defined($hRatedProductsStatistics) && $hRatedProductsStatistics->{'active'} > 0) {
        $Servlet->vars('CampaignCreationAllowed', 1);

        my $DefaultLanguageID = $Shop->get('LanguageID');
        my $DefaultCampaignName = $Shop->get('NameOrAlias', $DefaultLanguageID);
        $Servlet->vars('CampaignNameDefault', $DefaultCampaignName . " Campaign");

        my $hAdPreview = $self->_getAdPreview($Shop);
        $hAdPreview->{"shopName"} = $DefaultCampaignName;
        $hAdPreview->{"shopUrl"} = $Shop->get('WebUrlUnitySSL');
        $Servlet->vars('AdPreview', $hAdPreview);
   }

    return $self->SUPER::View($Servlet);
}

#========================================================================================
# §function     CreateGoogleAdsCampaign
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       CreateGoogleAdsCampaign( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Create a new Google Ads Campaign with the provided budget and an
#               associated conversion action.
#----------------------------------------------------------------------------------------
# §input        $Servlet | Presentation Object | object
#========================================================================================
sub CreateGoogleAdsCampaign {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $hFormValues = $Form->formValues();

    unless ($self->_areCampaignCreationRequirementsMet($Shop)) {
        $Form->executeFormError({
            'Name'   => 'CreationFailed',
            'Reason' => 'Campaign creation not possible for this Shop'
        });
    }

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    my $FormErrorReason = $self->_validateCreateGoogleAdsCampaignRequest($hFormValues, $GoogleSmartShoppingConfig);
    if (defined $FormErrorReason) {
        $Form->executeFormError({
            'Name'   => 'CreationFailed',
            'Reason' => $FormErrorReason
        });
    }

    my $GoogleAdsApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({Shop => $Shop});

    my $hAdsCampaignResponse = $self->_createCampaign($GoogleAdsApiService, $Shop, $GoogleSmartShoppingConfig, $Form);
    return unless( defined($hAdsCampaignResponse) && defined($hAdsCampaignResponse->{'id'}) );

    my $AdsCampaignId = $hAdsCampaignResponse->{'id'};
    my $hConversionActionResponse = $self->_createConversionAction($GoogleAdsApiService, $Form, $AdsCampaignId);
    return unless (
            defined($hConversionActionResponse) &&
            isValid($hConversionActionResponse->{'awConversionId'}) &&
            isValid($hConversionActionResponse->{'awConversionLabel'})
    );

    my $hRemarketingActionResponse = $self->_createRemarketingAction($GoogleAdsApiService, $Form, $AdsCampaignId);
    return unless (defined($hRemarketingActionResponse) && isValid($hRemarketingActionResponse->{'awRemarketingId'}));

    $GoogleSmartShoppingConfig->set({
        'AdsCampaignId' => $AdsCampaignId,
        'AdsCampaignConversionId' => $hConversionActionResponse->{'awConversionId'},
        'AdsCampaignConversionLabel' => $hConversionActionResponse->{'awConversionLabel'},
        'AdsCampaignRemarketingId' => $hRemarketingActionResponse->{'awRemarketingId'},
    });

    if ($hAdsCampaignResponse->{'status'} eq 'PAUSED') {
        $self->_enableCampaignStatus($GoogleAdsApiService, $Form, $AdsCampaignId);
    }

    LogGoogleSmartShopping('Activate ScripTag on created campaign (ID): ', $AdsCampaignId);
    my $ScriptTagService = DE_EPAGES::GoogleSmartShopping::API::ScriptTagService->new({Shop => $Shop});
    $ScriptTagService->addScriptTag();

    return;
}

#========================================================================================
# §function     GetGoogleAdsCampaign
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       GetGoogleAdsCampaign( $Shop, $GoogleSmartShoppingConfig );
#----------------------------------------------------------------------------------------
# §description  Get Smart Shopping Ads campaign
#----------------------------------------------------------------------------------------
# §input        $Shop | Shop Object | object
# §input        $GoogleSmartShoppingConfig | GoogleSmartShoppingConfig Object | object
#----------------------------------------------------------------------------------------
# §return       $hAdsCampaign | Smart Shopping Ads campaign | ref.hash
#========================================================================================
sub GetGoogleAdsCampaign {
    my $self = shift;
    my ($Shop, $GoogleSmartShoppingConfig) = @_;

    return unless $GoogleSmartShoppingConfig;
    my $AdsCampaignId = $GoogleSmartShoppingConfig->get('AdsCampaignId');
    return unless isValid($AdsCampaignId);

    my $service = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({Shop => $Shop});
    my $AdsSubAccountId = $GoogleSmartShoppingConfig->get('GoogleAdsAccountID');
    my $aAdsCampaigns;
    my $hParams = {
    'AdsSubAccountId' => $AdsSubAccountId,
    };
    eval {
        $aAdsCampaigns = $service->getSmartShoppingCampaigns($hParams);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Error getting AdsCampaigns: ', $Error->message());
        return { 'id' => $AdsCampaignId };
    }

    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
    my $CurrentDateTime = GetCurrentDateTime();
    foreach my $Campaign (@{$aAdsCampaigns}) {
        if ($Campaign->{'id'} eq $AdsCampaignId) {
            $Campaign->{'startDate'} = $Formatter->parse_datetime( $Campaign->{'startDate'}, 'date', '%Y-%m-%d' );
            if (DateTime->compare($Campaign->{'startDate'}, $CurrentDateTime) == 1) {
                $Campaign->{'isFuture'} = 1;
            }
            return $Campaign;
        }
    }

    LogGoogleSmartShopping('Error getting AdsCampaigns: ', 'Campaign id ' . $AdsCampaignId . ' not found in Google');
    return { 'id' => $AdsCampaignId };
}

#========================================================================================
# §function     UpdateGoogleAdsCampaign
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       UpdateGoogleAdsCampaign( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Update a Google Ads Campaign with the provided budget.
#----------------------------------------------------------------------------------------
# §input        $Servlet | Presentation Object | object
#========================================================================================
sub UpdateGoogleAdsCampaign {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    Error('Undefined configuration for GoogleSmartShopping') unless defined $GoogleSmartShoppingConfig;

    my $Form = $Servlet->form();
    my $hFormValues = $Form->formValues();
    LogGoogleSmartShopping('Google Ads Campaign update form values', { 'FormValues' => $hFormValues });

    my $FormErrorReason = $self->_validateUpdateGoogleAdsCampaignRequest($hFormValues, $GoogleSmartShoppingConfig);
    if (defined $FormErrorReason) {
        LogGoogleSmartShopping("Update Ads campaign validation error: ", $FormErrorReason);
        return;
    }

    my $DailyBudget = $self->_formatMoneyField($Shop, $Form, $hFormValues->{'DailyBudget'}->[0]);
    my $hCampaignBudgetUpdate = {
        'AdsCampaignId'       => $hFormValues->{'AdsCampaignId'}->[0],
        'DailyBudget'         => $DailyBudget,
        'DailyBudgetCurrency' => $hFormValues->{'DailyBudgetCurrency'}->[0]
    };

    my $GoogleAdsApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({Shop => $Shop});
    eval {
        $GoogleAdsApiService->updateGoogleAdsCampaignBudget($hCampaignBudgetUpdate);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Ads Campaign budget update failed', { 'Error' => $Error->message() });
        return;
    }

    return;
}

#========================================================================================
# §function     SwitchCampaignStatus
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       SwitchCampaignStatus( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Toggle campaign status between "ENABLED" and "PAUSED".
#----------------------------------------------------------------------------------------
# §input        $Servlet | Presentation Object | object
#========================================================================================
sub SwitchCampaignStatus {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    return unless ($self->_existsGoogleSmartShoppingConfig($Shop));

    my $Form = $Servlet->form();
    my $hPostData = $Form->formValues();

    my $CampaignId = $hPostData->{'CampaignId'}->[0];
    my $CampaignStatusEnabledString = $hPostData->{'CampaignStatusEnabled'}->[0];
    unless (isValid($CampaignStatusEnabledString)
        && ($CampaignStatusEnabledString ~~ ['true', 'false'])
        && isValid($CampaignId)) {
        $Servlet->response->code(HTTP_NOT_FOUND);
        return;
    }

    my $CampaignStatusEnabledValue;
    if ($CampaignStatusEnabledString eq 'true') {
        $CampaignStatusEnabledValue = 1;
    } else {
        $CampaignStatusEnabledValue = 0;
    }
    my $hCampaignStatusUpdate = {
        'CampaignId'            => $CampaignId,
        'CampaignStatusEnabled' => $CampaignStatusEnabledValue
    };
    my $GoogleAdsApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({ Shop => $Shop });
    my $ScriptTagService = DE_EPAGES::GoogleSmartShopping::API::ScriptTagService->new({Shop => $Shop});
    eval {
        $GoogleAdsApiService->updateGoogleAdsCampaign($hCampaignStatusUpdate);

        if ($hCampaignStatusUpdate->{'CampaignStatusEnabled'} ) {
            $ScriptTagService->addScriptTag();
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Ads Campaign status update failed', {
            'Error' => $Error->message(),
        });
        $Servlet->writeContentOutput({
        'Error' => $Error->message(),
        'CampaignId' => $CampaignId
        });
        return;
    }

    LogGoogleSmartShopping("Applied SwitchCampaignStatus: ", $hCampaignStatusUpdate);

    $Servlet->params('ContentType', 'application/json');
    $Servlet->response->code(HTTP_OK);
    $Servlet->writeContentOutput($hCampaignStatusUpdate);

    return;
}

#========================================================================================
# §function     DeleteAdsCampaign
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       DeleteAdsCampaign( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Delete a Google Ads Campaign.
#----------------------------------------------------------------------------------------
# §input        $Servlet | Presentation Object | object
#========================================================================================
sub DeleteAdsCampaign {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();

    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');
    Error('Undefined configuration for GoogleSmartShopping') unless defined $GoogleSmartShoppingConfig;

    my $Form = $Servlet->form();
    my $hFormValues = $Form->formValues();
    LogGoogleSmartShopping('Google Ads Campaign delete form values', { 'FormValues' => $hFormValues });

    my $FormErrorReason = $self->_validateDeleteAdsCampaignRequest($hFormValues, $GoogleSmartShoppingConfig);
    if (defined $FormErrorReason) {
        LogGoogleSmartShopping("Delete Ads campaign validation error: ", $FormErrorReason);
        $Form->executeFormError({
            'Name'   => 'DeletionFailed',
            'Reason' => 'Validation error'
        });
    }

    my $GoogleAdsApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService->new({Shop => $Shop});
    eval {
        $GoogleAdsApiService->deleteAdsCampaign($hFormValues->{'AdsCampaignId'}->[0]);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Delete Ads Campaign failed', { 'Error' => $Error->message() });
        $Form->executeFormError({
            'Name'   => 'DeletionFailed',
            'Reason' => GetErrorCode($Error)
        });
    }

    $GoogleSmartShoppingConfig->deleteAdsCampaign();
    LogGoogleSmartShopping("GoogleSmartShoppingConfig AdsCampaign deleted", {});

    return;
}


#----------------------------------------------------------------------------------------
sub _createCampaign {
    my $self = shift;
    my ($GoogleAdsApiService, $Shop, $GoogleSmartShoppingConfig, $Form) = @_;
    my $FormValues = $Form->values();

    my $AdsCampaignName = $FormValues->{'CampaignName'}->[0];
    my $AdsSubAccountId = $GoogleSmartShoppingConfig->get('GoogleAdsAccountID');
    my $MerchantCenterAccountId = $GoogleSmartShoppingConfig->get('MerchantCenterAccountID');

    my $DailyBudget = $self->_formatMoneyField($Shop, $Form, $FormValues->{'DailyBudget'}->[0]);

    my $hParams = {
        'AdsSubAccountId'         => $AdsSubAccountId,
        'MerchantCenterAccountId' => $MerchantCenterAccountId,
        'DailyBudget'             => $DailyBudget,
        'AdsCampaignName'         => $AdsCampaignName,
    };

    my $hAdsCampaignResponse;
    eval {
        $hAdsCampaignResponse = $GoogleAdsApiService->createGoogleAdsCampaign( $hParams );
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Ads Campaign creation failed', {
            'Error' => $Error->message(),
        });
        if ($Error->message() =~ /mutate_error: RESOURCE_NOT_FOUND/) {
            $GoogleSmartShoppingConfig->set({AreMerchantCenterAndAdsLinked => 0});
            $Form->executeFormError({
                'Name'   => 'CampaignCreationFailedNoMerchantCenterLink',
                'Reason' => GetErrorCode($Error)
            });
        }
        $Form->executeFormError({
            'Name'   => 'CampaignCreationFailed',
            'Reason' => GetErrorCode($Error)
        });
    }

    return $hAdsCampaignResponse;
}

#----------------------------------------------------------------------------------------
sub _createConversionAction {
    my $self = shift;
    my ($GoogleAdsApiService, $Form, $AdsCampaignId) = @_;

    my $hResponse;
    eval {
        $hResponse = $GoogleAdsApiService->createGoogleConversionAction( $AdsCampaignId );
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Ads Campaign conversion action creation failed', {
            'Error' => $Error->message(),
        });
        $Form->executeFormError({
            'Name'   => 'ConversionActionCreationFailed',
            'Reason' => GetErrorCode($Error)
        });
    }

    return $hResponse;
}
#----------------------------------------------------------------------------------------
sub _createRemarketingAction {
    my $self = shift;
    my ($GoogleAdsApiService, $Form, $AdsCampaignId) = @_;

    my $hResponse;
    eval {
        $hResponse = $GoogleAdsApiService->createGoogleRemarketingAction($AdsCampaignId);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Ads Campaign remarketing action creation failed', {
            'Error' => $Error->message(),
        });
        $Form->executeFormError({
            'Name'   => 'RemarketingActionCreationFailed',
            'Reason' => GetErrorCode($Error)
        });
    }

    return $hResponse;
}

#----------------------------------------------------------------------------------------
sub _areCampaignCreationRequirementsMet {
    my $self = shift;
    my ($Shop, $hProductStats) = @_;

    return unless ($self->_existsGoogleSmartShoppingConfig($Shop));
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');

    return unless ($GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());
    # most important is that we have both Merchant Center and Ads accounts connected
    return unless (isValid($GoogleSmartShoppingConfig->get('MerchantCenterAccountID')));
    return unless (isValid($GoogleSmartShoppingConfig->get('GoogleAdsAccountID')));
    return unless ($GoogleSmartShoppingConfig->get('AreMerchantCenterAndAdsLinked'));

    return 1;
}

#----------------------------------------------------------------------------------------
sub _validateDeleteAdsCampaignRequest {
    my $self = shift;
    my ($hFormValues, $GoogleSmartShoppingConfig) = @_;

    my $ValidationError = $self->_validateCampaignRequest($hFormValues, $GoogleSmartShoppingConfig);
    return $ValidationError if (defined($ValidationError));

    return ERROR_FORM_SUBMIT unless (
        isValid($hFormValues->{'AdsCampaignId'}) && isValid($hFormValues->{'AdsCampaignId'}->[0])
        && ($hFormValues->{'AdsCampaignId'}->[0] eq $GoogleSmartShoppingConfig->get('AdsCampaignId'))
    );

    return;
}

#----------------------------------------------------------------------------------------
sub _validateCreateGoogleAdsCampaignRequest {
    my $self = shift;
    my ($hFormValues, $GoogleSmartShoppingConfig) = @_;

    my $ValidationError = $self->_validateCampaignRequest($hFormValues, $GoogleSmartShoppingConfig);
    return $ValidationError if (defined($ValidationError));

    return ERROR_FORM_SUBMIT unless (
        isValid($hFormValues->{'DailyBudget'}) && isValid($hFormValues->{'DailyBudget'}->[0])
    );
    return;
}

#----------------------------------------------------------------------------------------
sub _validateUpdateGoogleAdsCampaignRequest {
    my $self = shift;
    my ($hFormValues, $GoogleSmartShoppingConfig) = @_;

    my $ValidationError = $self->_validateCreateGoogleAdsCampaignRequest($hFormValues, $GoogleSmartShoppingConfig);
    return $ValidationError if (defined($ValidationError));

    return ERROR_FORM_SUBMIT unless (
        isValid($hFormValues->{'AdsCampaignId'}) && isValid($hFormValues->{'AdsCampaignId'}->[0])
    );
    return ERROR_FORM_SUBMIT unless (
        isValid($hFormValues->{'DailyBudgetCurrency'}) && isValid($hFormValues->{'DailyBudgetCurrency'}->[0])
    );

    return;
}

#----------------------------------------------------------------------------------------
sub _validateCampaignRequest {
    my $self = shift;
    my ($hFormValues, $GoogleSmartShoppingConfig) = @_;

    return ERROR_ACCEPT_TERMS unless ($GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());
    return ERROR_VALIDATION unless (isValid($GoogleSmartShoppingConfig->get('GoogleAdsAccountID')));
    return ERROR_VALIDATION unless (isValid($GoogleSmartShoppingConfig->get('MerchantCenterAccountID')));

    return;
}

#----------------------------------------------------------------------------------------
sub _existsGoogleSmartShoppingConfig {
    my $self = shift;
    my ($Shop) = @_;

    my $CallingFunction = ( caller(0) )[3];
    unless ($Shop->existsChild('GoogleSmartShoppingConfig')) {
        LogGoogleSmartShopping("GoogleSmartShoppingConfig shop object not found in ", $CallingFunction);
        return 0;
    }
    return 1;
}

sub _formatMoneyField {
    my $self = shift;
    my ($Shop, $Form, $FieldValue) = @_;

    my $hFieldData = {
        'Name'       => 'Amount',
        'Type'       => 'reg_money',
        'Mandatory'  => 1,
        'MinValue'   => 0,
        'MaxValue'   => undef,
        'CurrencyID' => $Shop->get('CurrencyID')
    };

    return $Form->field($Shop, $FieldValue, $hFieldData);
}

#--------------------------------------------------------------------------------------------

sub _getRatedProductStatistics {
    my $self = shift;
    my ($Shop) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    my $hRatedProductsStatistics;
    eval {
        $hRatedProductsStatistics = $Service->getRatedProductsStatistics();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping("Google ProductLevelIssues request failed: ", {
            Error => $Error,
            ErrorMessage => $Error->message(), 
        });
    }
    return $hRatedProductsStatistics;
}

#--------------------------------------------------------------------------------------------

sub _getAdPreview {
    my $self = shift;
    my ($Shop) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});
    my $hAdPreview;

    eval {
        $hAdPreview = $Service->getFirstProduct();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping("Google getAdPreview (products.list) request failed: ", {
            Error => $Error,
            ErrorMessage => $Error->message(),
        });
    }

    return $hAdPreview;
}

#----------------------------------------------------------------------------------------
sub _enableCampaignStatus {
    my $self = shift;
    my ($GoogleAdsApiService, $Form, $AdsCampaignId) = @_;

    my $hCampaignStatusUpdate = {
        'CampaignId'            => $AdsCampaignId,
        'CampaignStatusEnabled' => 1
    };
    eval {
        $GoogleAdsApiService->updateGoogleAdsCampaign($hCampaignStatusUpdate);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Ads Campaign status update failed', {
            'Error' => $Error->message(),
        });
        $Form->executeFormError({
            'Name'   => 'CampaignStatusUpdateFailed',
            'Reason' => GetErrorCode($Error)
        });
    }
    LogGoogleSmartShopping("Campaign status enabled: ", $hCampaignStatusUpdate);

    return;
}

sub _getPortalConfig {
    my $self = shift;
    my ($Shop) = @_;
    my $GooglePortalService = DE_EPAGES::GoogleSmartShopping::API::GooglePortalService->new({ Shop => $Shop });
    return $GooglePortalService->getSmartShoppingPortalConfig();
}

1;
