#==============================================================================
# package      DE_EPAGES::GoogleSmartShopping::UI::ProductFeed
# state        private
#==============================================================================
package DE_EPAGES::GoogleSmartShopping::UI::ProductFeed;
use base DE_EPAGES::ProductPortal::UI::PortalConfig;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use HTTP::Status qw ( HTTP_OK );

use DE_EPAGES::GoogleSmartShopping::API::Tools qw ( LogGoogleSmartShopping );
use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;
use DE_EPAGES::GoogleSmartShopping::API::GooglePortalService;
use DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts;
use DE_EPAGES::Product::API::Object::PagerProducts;

use strict;

#==============================================================================
# Event handlers
#==============================================================================
sub UnityViewGoogleSmartShoppingProductFeed {
    my $self = shift;
    my ($Servlet) = @_;

    my $GoogleSmartShoppingPortalConfig = $Servlet->object();
    my $Shop = $GoogleSmartShoppingPortalConfig->get('Shop');

    my $IsAllowedProductUpload = $self->_areProductUploadRequirementsMet($Shop);
    $Servlet->vars('IsAllowedProductUpload', $IsAllowedProductUpload);

    unless (defined $IsAllowedProductUpload) {
        LogGoogleSmartShopping('Product Upload not allowed', 1);
        return $self->SUPER::View($Servlet);
    }

    my $hProductStatuses = $self->_getProductStatistics($Shop);
    my $aPortalProducts = $GoogleSmartShoppingPortalConfig->get('Products');

    $Servlet->vars('ProductsInPortalConfig', scalar(@$aPortalProducts));
    $Servlet->vars('ProductStatuses', $hProductStatuses);

    return $self->_viewPagedChildren($Servlet);
}

sub _viewPagedChildren {
    my $self = shift;
    my ($Servlet) = @_;

    my $PortalConfig = $Servlet->object();
    my $Shop = $PortalConfig->getSite();
    my $GoogleApiService = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({Shop => $Shop});

    my $Pager = DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts->new({
        'Servlet' => $Servlet,
        'Object'  => $PortalConfig,
        'GoogleApiService' => $GoogleApiService
    });

    $Pager->execute();
    return $Pager->page;

}

# this is a stub
sub RemoveProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form();
    my $hPostData = $Form->formValues();

    #return $self->SUPER::RemoveProducts($Servlet);
    LogGoogleSmartShopping("Remove selected productsIDs ", {
        PostData => $hPostData,
    });
}

# this is a stub
sub RemoveAllProducts {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->SUPER::RemoveAllProducts($Servlet);
}

# This change action sets the flag to export all products of the Shop to Google
sub UploadAllProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $PortalConfig = $Servlet->object();

    my $aProducts = $PortalConfig->get('Products');
    if (scalar @{$aProducts} > 0)  {
        $PortalConfig->removeAllProducts();
    }

    $PortalConfig->set({'ExportAllProducts', 1});

    $self->SUPER::GeneratePortalConfigProductPrettyUnityUrls($Servlet);

    return $self->SUPER::ExportPortalConfig($Servlet, 0);
}

#========================================================================================
# function     ExportPortalConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       ExportPortalConfig( $Servlet );
#----------------------------------------------------------------------------------------
# description  ExportPortalConfig uses the parent classes to starts the export process
#               that gathers product info and send it to an API
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet Object | object
#========================================================================================
sub ExportPortalConfig {
    my $self    = shift;
    my ($Servlet) = @_;

    LogGoogleSmartShopping("ExportPortalConfig initiated", {});

    my $PortalConfig = $Servlet->object();

    if ($PortalConfig->get('ExportAllProducts'))  {
        $PortalConfig->set({'ExportAllProducts', 0});
    }

    $self->SUPER::GeneratePortalConfigProductPrettyUnityUrls($Servlet);

    return $self->SUPER::ExportPortalConfig($Servlet, 0);
}

#==============================================================================
sub _areProductUploadRequirementsMet {
    my $self = shift;
    my ($Shop) = @_;

    return unless (defined($Shop->get('Address')->get('Country')));
    return unless ($Shop->existsChild('GoogleSmartShoppingConfig'));
    my $GoogleSmartShoppingConfig = $Shop->get('GoogleSmartShoppingConfig');

    return unless ($GoogleSmartShoppingConfig->areGoogleMerchantCenterRequirementsMet());
    # most important is that we have Merchant Center account connected
    return unless (isValid($GoogleSmartShoppingConfig->get('MerchantCenterAccountID')));

    return 1;
}

sub _getProductStatistics {
    my $self = shift;
    my ($Shop) = @_;

    my $Service = DE_EPAGES::GoogleSmartShopping::API::GoogleApiService->new({ Shop => $Shop });
    my $hResult;
    eval {
        $hResult = $Service->getRatedProductsStatistics();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping("Google ProductLevelIssues request failed: ", {
            Error        => $Error,
            ErrorMessage => $Error->message(),
        });
    }
    return $hResult;
}

sub ChangePortalShippingMethod {
    my $self = shift;
    my ($Servlet) = @_;

    my $PortalConfig = $Servlet->object();
    my $Shop = $PortalConfig->get('Shop');

    my $SelectedShippingMethodId = $self->_getSelectedShippingMethodID($Servlet);
    LogGoogleSmartShopping('ChangePortalShippingMethod: New Shipping method from Template', $SelectedShippingMethodId);

    my $ShippingMethodId = $self->_setShippingMethod($Shop, $PortalConfig, $SelectedShippingMethodId);
    LogGoogleSmartShopping("ChangePortalShippingMethod: shipping Method set in Database", $ShippingMethodId);

    $Servlet->params('ContentType', 'application/json');
    $Servlet->response->code(HTTP_OK);
    $Servlet->writeContentOutput({'ShippingMethodID' => $ShippingMethodId});
}

sub _getSelectedShippingMethodID {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form();
    my $hPostData = $Form->formValues();

    return $hPostData->{'selectedShippingMethodId'}->[0];
}

sub _setShippingMethod {
    my $self = shift;
    my ($Shop, $PortalConfig, $ShippingMethodId) = @_;
    my $aProducts = $PortalConfig->get('Products');
    if (scalar @{$aProducts} == 0)  {
        $self->_storeShippingMethodInPortalConfig($Shop, $PortalConfig, $ShippingMethodId);
    }
    else {
        my $ShippingMethod = $PortalConfig->get('ShippingMethod');
        $ShippingMethodId = $ShippingMethod->get('ID');
    }
    return $ShippingMethodId;
}

sub _storeShippingMethodInPortalConfig {
    my $self = shift;
    my ($Shop, $PortalConfig, $ShippingMethodId) = @_;
    my $ShippingMethod = $Shop->loadObject($ShippingMethodId, 'ShippingMethod');
    eval {
        TestObject('ShippingMethod', $ShippingMethod, 'DE_EPAGES::Shipping::API::Object::ShippingMethod');
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogGoogleSmartShopping('Error getting AdsCampaigns: ', $Error->message());
    }
    $PortalConfig->set({
        'ShippingMethod'  => $ShippingMethod,
    });
    return;
}
1;
