#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::GoogleSmartShoppingProductsExportDriver
# base         DE_EPAGES::Core::API::Test
#----------------------------------------------------------------------------------------
# description  unit test for GoogleSmartShoppingProductsExportDriver
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::GoogleSmartShoppingProductsExportDriver;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::GoogleSmartShopping::API::REST::GoogleSmartShoppingProductsExportDriver;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use Test::Mockify;
use Test::Mockify::Matcher qw ( Object Number );
use Test::Mockify::Verify qw (  WasCalled );

use strict;

sub test {
    my $self = shift;

    $self->test_google_product_category_Uppercase_export();
    $self->test_google_product_category_Lowercase_export();
    $self->test_google_product_category_export_emptyValue();
    $self->test_google_product_category_removeUnexpectedCharacters();
    $self->test_google_shipping_productShippingEqualsPortalShipping();
    $self->test_google_shipping_productShippingNotEqualsPortalShipping();

    $self->test_offer_id();
    $self->test_description();
    $self->test_title();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_google_product_category_Uppercase_export {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Google_Product_Category' => 'TestCategory',
        'ProductTypeAttributes' => [
            'Google_Product_Category',
        ]
    });

    my $Result = $SUT->googleProductCategory( $Product );

    is( $Result, 'TestCategory', "$SubTestName - prove that the google category is exported." );
}
#----------------------------------------------------------------------------------------
sub test_google_product_category_Lowercase_export {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'google_product_category' => 'TestUnityCategory',
        'ProductTypeAttributes' => [
            'google_product_category',
        ]
    });

    my $Result = $SUT->googleProductCategory( $Product );

    is( $Result, 'TestUnityCategory', "$SubTestName - prove that the google category is exported." );
}
#----------------------------------------------------------------------------------------
sub test_google_product_category_export_emptyValue {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Google_Product_Category' => undef,
        'google_product_category' => undef,
        'ProductTypeAttributes' => [
            'Google_Product_Category',
            'google_product_category',
        ]
    });

    my $Result = $SUT->googleProductCategory( $Product );

    is($Result, undef, "$SubTestName - prove that the google category is exporting undef.");
}


#----------------------------------------------------------------------------------------
sub test_google_product_category_removeUnexpectedCharacters {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Google_Product_Category' => "\"Test\" Category\n second Line",
        'ProductTypeAttributes' => [
            'Google_Product_Category',
        ]
    });

    my $Result = $SUT->googleProductCategory( $Product );

    is( $Result, 'Test Category', "$SubTestName - prove that the google category is exported." );
    is( WasCalled($SUT, 'Warning'), 0, "$SubTestName - prove that no warning was created." );
}
#----------------------------------------------------------------------------------------
sub test_google_shipping_productShippingEqualsPortalShipping {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct({
        'Google_Product_Category' => "\"Test\" Category\n second Line",
        'ProductTypeAttributes' => [
            'Google_Product_Category',
        ]
    });

    my $hFormattedShippingPrice = {
        currency => 'EUR',
        value => 1
    };

    my $hProductShippingAttributes = {
        'FormattedShippingPrice' => $hFormattedShippingPrice,
        'ShippingMethodId' => 123,
    };

    my $PortalShippingMethod = $self->_createShippingMethod({'ShippingMethodId' => 123});

    my $SUT = $self->_createSUT($hProductShippingAttributes, $PortalShippingMethod);

    $SUT->{'PortalSiteUppercaseCountry'} = 'DE';
    $SUT->{'PortalShippingMethodCountries'} = ['TH'];

    my $ActualResult = $SUT->shipping( $Product );
    my $ExpectedResult = [{
        country => 'TH',
        price => $hFormattedShippingPrice,
    }];

    is_deeply( $ActualResult, $ExpectedResult, "$SubTestName - asserts that default shipping countries are exported" );
    is( WasCalled($SUT, '_getShippingMethodCountries'), 0, "$SubTestName - asserts that _getShippingMethodCountries was not called." );
}

sub test_google_shipping_productShippingNotEqualsPortalShipping {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct({
        'Google_Product_Category' => "\"Test\" Category\n second Line",
        'ProductTypeAttributes' => [
            'Google_Product_Category',
        ]
    });

    my $aRegionCountries = [
        {'CountryID' => 1, 'Code2' => 'DE'},
        {'CountryID' => 2, 'Code2' => 'AT'},
        {'CountryID' => 3, 'Code2' => 'CH'}
    ];

    my $hFormattedShippingPrice = {
        currency => 'EUR',
        value => 2
    };

    my $hProductShippingAttributes = {
        'FormattedShippingPrice' => $hFormattedShippingPrice,
        'ShippingMethodId' => 321,
        'Region' => $aRegionCountries
    };

    my $PortalShippingMethod = $self->_createShippingMethod({'ShippingMethodId' => 123});

    my $SUT = $self->_createSUT($hProductShippingAttributes, $PortalShippingMethod);

    $SUT->{'PortalSiteCountryID'} = 1;
    $SUT->{'PortalSiteUppercaseCountry'} = 'DE';
    $SUT->{'PortalShippingMethodCountries'} = ['TH'];

    my $ActualResult = $SUT->shipping( $Product );
    my $ExpectedResult = [
        {country => 'DE',price => $hFormattedShippingPrice},
        {country => 'AT',price => $hFormattedShippingPrice},
        {country => 'CH',price => $hFormattedShippingPrice},
    ];

    is_deeply( $ActualResult, $ExpectedResult, "$SubTestName - asserts that product shipping countries are exported" );
    ok( WasCalled($SUT, '_getShippingMethodCountries'), "$SubTestName - asserts that _getShippingMethodCountries was called." );
}

sub test_offer_id {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Alias' => 'Test Alias',
    });

    my $Result = $SUT->offerId( $Product );

    is( $Result, 'Test Alias_en_GB', "$SubTestName - prove that the offerId is exported." );
}

sub test_description {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Alias' => 'Test Alias',
    });

    my $Result = $SUT->offerId( $Product );

    is( $Result, 'Test Alias_en_GB', "$SubTestName - prove that the offerId is exported." );
}

sub test_title {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Name' => 'Test Name',
    });

    my $Result = $SUT->title( $Product );

    is( $Result, 'Test Name', "$SubTestName - prove that the offerId is exported." );
    is( WasCalled($SUT, 'Warning'), 0, "$SubTestName - prove that no warning was created." );
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hAttributes, $PortalShippingMethod) = @_;

    my $aParameterList = [];
    my $Mockify = Test::Mockify->new(
        'DE_EPAGES::GoogleSmartShopping::API::REST::GoogleSmartShoppingProductsExportDriver',
        $aParameterList
    );

    $Mockify->spy('Warning')->whenAny();
    $Mockify->spy('_getShippingMethodCountries')->when(Object());
    $Mockify->mock('object')->when()->thenReturn($self->_createPortalConfig($hAttributes));
    $Mockify->mock('shippingPrice')->when(Object())->thenReturn(1);
    $Mockify->mock('formatPrice')->when(Number(1))->thenReturn($hAttributes->{'FormattedShippingPrice'} // 1);
    $Mockify->mock('shippingMethod')->when()->thenReturn($PortalShippingMethod // 1);

    $Mockify->mock('portalSite')->when()->thenReturn($self->_createPortalSite());

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createProduct {
    my $self = shift;
    my ($hValues) = @_;

    my $hAttributes = {
        'ID' => '666',
        'Alias' => $hValues->{'Alias'} // undef,
        'Name' => $hValues->{'Name'} // undef,
        'Description' => $hValues->{'Alias'} // undef,
        'Google_Product_Category' => $hValues->{'Google_Product_Category'} // undef,
        'google_product_category' => $hValues->{'google_product_category'} // undef,
    };

    if ($hValues->{'ProductTypeAttributes'}) {
        $hAttributes->{'Class'} = $self->_createProductType($hValues->{'ProductTypeAttributes'});
    }

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product',
        $hAttributes
    );

    if ($hValues->{'Name'}) {
        $Mockify->addMockWithReturnValueAndParameterCheck('getWithParent', $hValues->{'Name'}, [{'string' => 'Name'}, {'string'}]);
    }

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createProductType {
    my $self = shift;
    my ($aDirectAttributes) = @_;

    my $hAttributes = {
        'Alias' => 'MyShop',
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::ProductType',
        $hAttributes
    );
    my $Product = $Mockify->getMockObject();
    #This is a hack in order to get the corret result when calling existsAttribute on this class
    foreach my $DirectAttribute (@{$aDirectAttributes}){
        $Product->{'DirectAttributes'}->{$DirectAttribute} = 1;
    }
    return $Product;
}
#----------------------------------------------------------------------------------------
sub _createPortalConfig {
    my $self = shift;
    my ($hAttributes) = @_;

    my $hParameterList = {
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::ProductPortal::API::Object::PortalConfig',
            $hParameterList
    );

    $Mockify->addMockWithReturnValueAndParameterCheck('shippingMethodForProduct', $self->_createShippingMethod($hAttributes), ['object']);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRegion {
    my $self = shift;
    my ($hAttributes) = @_;

    if (!defined $hAttributes->{'Region'}) {
        return;
    }

    my $hRegionAttributes = {
        'Countries' => $hAttributes->{'Region'}
    };

    my $Mockify =
      DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Region', $hRegionAttributes);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShippingMethod {
    my $self = shift;
    my ($hAttributes) = @_;

    my $hShippingAttributes = {
        'ID' => $hAttributes->{'ShippingMethodId'},
        'Region' => $self->_createRegion($hAttributes),
    };

    my $Mockify =
      DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shipping::API::Object::ShippingMethod', $hShippingAttributes);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPortalSite {
    my $self = shift;

    my $hAttributes = {
        'LocaleID' => 'en_GB',
        'Alias' => 'GoogleSmartShopping'
    };

    my $MockifyPortalSite = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalSite', $hAttributes
    );

    return $MockifyPortalSite->getMockObject();
}
__PACKAGE__->runScript();
