#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::ProductUploadService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for API::ProductUploadService
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::ProductUploadService;
use base DE_EPAGES::Core::API::Test;


use DE_EPAGES::GoogleSmartShopping::API::GoogleApiService;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify;
use Test::Mockify::Sut;
use Test::Exception;
use Test::MockObject::Extends;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::Mockify::Matcher qw ( String HashRef );
use Test::More;

use strict;

use constant LOCALE_ID => 'en_GB';

use DE_EPAGES::GoogleSmartShopping::API::Constants qw ( NUMBER_PRODUCTS_REQUEST_THRESHOLD );

use constant PRODUCT_EXPORT_DATA => {
    "link" => "https://demoldhgst4tgga.vm-hsilva.intern.epages.de/p/unisex-sneakers-brombeer",
    "isBundle" => "false",
    "productTypes" => ["Schuhe"],
    "identifierExists" => "false",
    "description" => "The modern alternative to the classic lace-up sneakers. An easy-on design with a subtle print and elastic inserts for a comfortable and flexible fit. The perfect update to your wardrobe! Animal print in firm textile Off-white rubber sole Fylly leather inside Cushioned insole Removable insole",
    "availability" => "in stock",
    "offerId" => "56-sneakers_de_DE",
    "title" => "Unisex sneakers - brombeer",
    "imageLink" => "http://demoldhgst4tgga.vm-hsilva.intern.epages.de/WebRoot/Store/Shops/demoldhgst4tgga/Products/56-sneakers/sneaker-classic-brombeer.jpg",
    "price" => {
        "currency" => "EUR",
        "value" => "79.90"
    },
    "shipping" => [
        {
            "country" => "DE",
            "price" => {
                "currency" => "EUR",
                "value" => "7.00"
            }
         }
    ],
    "condition" => "new",
};

sub test {
    my $self = shift;

    $self->test_OnStartExport();
    $self->test_ExportItem_Less_than_Threshold();
    $self->test_ExportItem_Bigger_than_Threshold();
    $self->test_OnEndExport_empty_products();
    $self->test_OnEndExport_existing_products();

    done_testing();
}

sub test_OnStartExport {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $MockedShop = $self->_createShop();

    my $MockedGoogleConfig = $self->_createGoogleConfig();
    my $MockedGoogleApiService = $self->_createGoogleApiService($MockedShop, $MockedGoogleConfig);
    my $SUT = $self->_createSUT($MockedShop, $MockedGoogleApiService);

    lives_ok(
        sub {
            $SUT->OnStartExport();
        },
        "$SubTestName - OnStartExport can be called without errors."
    );

    ok(
        WasCalled($MockedGoogleApiService, 'loadGoogleSettings'),
        $SubTestName . ' - Verify that loadGoogleSettings was called.'
    );

    ok(
        WasCalled($MockedGoogleConfig, 'get'),
        $SubTestName . ' - Verify that get MerchantCenterAccountID was called.'
    );

    is_deeply(
        $SUT->{'GoogleApiService'},
        $MockedGoogleApiService,
        $SubTestName . ' - Verify that GoogleApiService was set up correctly'
    );

    is(
        $SUT->{'MerchantCenterAccountID'},
        12345,
        $SubTestName . ' - Verify that MerchantCenterAccountID was set up correctly'
    );

    is_deeply($SUT->{'Products'}, [], $SubTestName . ' - Verify that Products array is empty');
}

sub test_ExportItem_Less_than_Threshold{
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $MockedShop = $self->_createShop();

    my $SUT = $self->_createSUT($MockedShop);

    $SUT->OnStartExport();

    lives_ok(
        sub {
            $SUT->ExportItem(PRODUCT_EXPORT_DATA);
        },
        "$SubTestName - ExportItem can be called without errors."
    );

    is(@{$SUT->{'Products'}}, 1, $SubTestName . ' - Verify that Products array has 1 product');

    ok(
        !WasCalled($SUT, '_sendInsertBatchRequest'),
        $SubTestName . ' - Verify that _sendInsertBatchRequest was NOT called.'
    );

    lives_ok(
        sub {
            for (my $i = 0; $i < NUMBER_PRODUCTS_REQUEST_THRESHOLD - 2; $i++) {
                $SUT->ExportItem(PRODUCT_EXPORT_DATA);
            }
        },
        "$SubTestName - ExportItem can be called (NUMBER_PRODUCTS_REQUEST_THRESHOLD - 2) without errors."
    );

    ok(
        !WasCalled($SUT, '_sendInsertBatchRequest'),
        $SubTestName . ' - Verify that _sendInsertBatchRequest was NOT called.'
    );

    is(scalar @{$SUT->{'Products'}}, NUMBER_PRODUCTS_REQUEST_THRESHOLD - 1, $SubTestName . ' - Verify that Products array is (NUMBER_PRODUCTS_REQUEST_THRESHOLD - 1)');
}

sub test_ExportItem_Bigger_than_Threshold{
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];
    

    my $MockedGoogleConfig = $self->_createGoogleConfig();
    my $MockedShop = $self->_createShop();
    my $MockedGoogleApiService = $self->_createGoogleApiService($MockedShop, $MockedGoogleConfig);

    my $SUT = $self->_createSUT($MockedShop, $MockedGoogleApiService);

    $SUT->OnStartExport();

    lives_ok(
        sub {
            for (my $i = 0; $i < NUMBER_PRODUCTS_REQUEST_THRESHOLD; $i++) {
                $SUT->ExportItem(PRODUCT_EXPORT_DATA);
            }
        },
        "$SubTestName - ExportItem can be called n amount of times without errors."
    );


    ok(
        WasCalled($SUT, '_sendInsertBatchRequest'),
        $SubTestName . ' - Verify that _sendInsertBatchRequest was called.'
    );

    ok(
        WasCalled($SUT->{'GoogleApiService'}, 'callProductsBatch'),
        $SubTestName . ' - Verify that callProductsBatch was called.'
    );

    my $aCallProductsBatchParams = GetParametersFromMockifyCall($SUT->{'GoogleApiService'}, 'callProductsBatch');

    is_deeply(
        $aCallProductsBatchParams->[0]->{'entries'}->[0]->{'product'},
        PRODUCT_EXPORT_DATA,
        $SubTestName . ' - Verify that callProductsBatch has correct parameters on first product'
    );

    is_deeply(
        $aCallProductsBatchParams->[0]->{'entries'}->[NUMBER_PRODUCTS_REQUEST_THRESHOLD - 1]->{'product'},
        PRODUCT_EXPORT_DATA,
        $SubTestName . ' - Verify that callProductsBatch has correct parameters on last product'
    );

    is_deeply($SUT->{'Products'}, [], $SubTestName . ' - Verify that Products array is emptied after upload');
}

sub test_OnEndExport_empty_products {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $MockedShop = $self->_createShop();

    my $SUT = $self->_createSUT($MockedShop);

    $SUT->OnStartExport();

    is_deeply($SUT->{'Products'}, [], $SubTestName . ' - Verify that Products array is empty');

    lives_ok(
        sub {
            $SUT->OnEndExport();
        },
        "$SubTestName - OnEndExport can be called without errors."
    );

    ok(
        !WasCalled($SUT, '_sendInsertBatchRequest'),
        $SubTestName . ' - Verify that _sendInsertBatchRequest was NOT called when empty Product array.'
    );

}

sub test_OnEndExport_existing_products {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $MockedShop = $self->_createShop();

    my $SUT = $self->_createSUT($MockedShop);

    $SUT->OnStartExport();

    $SUT->ExportItem(PRODUCT_EXPORT_DATA);

    is(@{$SUT->{'Products'}}, 1, $SubTestName . ' - Verify that Products array has 1 product');

    lives_ok(
        sub {
            $SUT->OnEndExport();
        },
        "$SubTestName - OnEndExport can be called without errors."
    );


    ok(
        WasCalled($SUT, '_sendInsertBatchRequest'),
        $SubTestName . ' - Verify that _sendInsertBatchRequest was called.'
    );

    ok(
        WasCalled($SUT->{'GoogleApiService'}, 'callProductsBatch'),
        $SubTestName . ' - Verify that callProductsBatch was called.'
    );

    my $aCallProductsBatchParams = GetParametersFromMockifyCall($SUT->{'GoogleApiService'}, 'callProductsBatch');

    is_deeply(
        $aCallProductsBatchParams->[0]->{'entries'}->[0]->{'product'},
        PRODUCT_EXPORT_DATA,
        $SubTestName . ' - Verify that callProductsBatch has correct parameters on first product'
    );

    is_deeply($SUT->{'Products'}, [], $SubTestName . ' - Verify that Products array is emptied after upload');
}

sub _createSUT {
    my $self = shift;
    my ($MockedShop, $MockedGoogleApiService) = @_;

    my $MockifySUT = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::API::ProductUploadService', [$MockedShop]);

    $MockifySUT->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    if (defined($MockedGoogleApiService)) {
        $MockifySUT->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when(String(), HashRef())
            ->thenReturn($MockedGoogleApiService);
    }
    
    $MockifySUT->spy('_sendInsertBatchRequest')->when();

    return $MockifySUT->getMockObject();
}

sub _createGoogleApiService {
    my $self = shift;
    my ($MockedShop, $MockedGoogleConfig) = @_;

    my $hServiceParams = {
        'Shop' => $MockedShop,
        'GoogleSmartShoppingConfig' => $MockedGoogleConfig,
    };

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService', [$hServiceParams]);

    $Mockify->mock('loadGoogleSettings')->when()->thenReturnUndef();

    $Mockify->mock('callProductsBatch')->when(HashRef())->thenReturnUndef();

    my $MockedGoogleApiService = $Mockify->getMockObject();
    $MockedGoogleApiService->{'GoogleSmartShoppingConfig'} = $MockedGoogleConfig;

    return $MockedGoogleApiService;
}

sub _createGoogleConfig {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig');

    $Mockify->mock('get')->when(String('MerchantCenterAccountID'))->thenReturn(12345);

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'ID'            => undef,
        'LocaleID'      => LOCALE_ID,
        'LanguageID'    => 1,
        'NameOrAlias'   => 'ShopNameOrAlias',
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    return $MockifyShop->getMockObject();
}


__PACKAGE__->runScript();

1;
