#========================================================================================
# package      DE_EPAGES::Presentation::API::REST::MonitoredExportDriver
# base         DE_EPAGES::Core::API::REST::BaseExportDriver
# state        public
#----------------------------------------------------------------------------------------
# description  This a base for your servlet driven exports of comma-separated files.
#               Overwrite the <function drive> function to implement a custom CVS file
#               format.
#
# example      use DE_EPAGES::Presentation::API::REST::MonitoredExportDriver;
#               my $Driver = DE_EPAGES::Presentation::API::REST::MonitoredExportDriver->new;
#               my $Class = LoadClassByAlias('Product');
#               my @Columns = map { $Class->attribute($_) } qw( Alias Name );
#               $Driver->exportREST( $FileName, {}, \@Columns, $Shop, $aObjects->get('Products') );
#========================================================================================
package DE_EPAGES::Presentation::API::REST::MonitoredExportDriver;
use base DE_EPAGES::Core::API::REST::BaseExportDriver;

use strict;
use DE_EPAGES::Presentation::API::Object::ProgressMonitor;

#========================================================================================
# function     exportREST
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exporter->exportREST($FileName, $hFormat, $aColumns, $ViewObject, $aObjects );
#----------------------------------------------------------------------------------------
# description  Exports objects to a comma-separated file.
#----------------------------------------------------------------------------------------
# input        $FileName |  export file | string
# input        $hFormat | export file format
#                       <ul>
#                         <li>see: <function DE_EPAGES::Core::API::REST::BaseExportDriver::importREST></li>
#                         <li>Monitor - progress monitor (optional, see: <package DE_EPAGES::Presentation::API::Object::ProgressMonitor>)</li>
#                       </ul> | ref.hash
# input        $aColumns | column attributes (used for header and values) | attribute object
# input        $ViewObject | object to view monitor | object
# input        $aObjects | list of objects | ref.array.object
#========================================================================================
sub exportREST {
    my $self = shift;
    my ($aColumns, $ViewObject, $aObjects) = @_;

    $self->{'Object'} = $ViewObject;
    return $self->SUPER::exportREST($aColumns, $aObjects);
}

#========================================================================================
# function     monitor
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Monitor = $self->monitor;
#----------------------------------------------------------------------------------------
# description  Returns the progress monitor object.
#----------------------------------------------------------------------------------------
# return       $Monitor  | progress monitor | object
#========================================================================================
sub monitor { return shift->{'Monitor'}; }

#========================================================================================
# function     object
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $self->object;
#----------------------------------------------------------------------------------------
# description  Return the current view object (central object for exported objects).
#----------------------------------------------------------------------------------------
# return       $Object | object | object
#========================================================================================
sub object { return shift->{'Object'}; }

#========================================================================================
# function     drive
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->drive();
#----------------------------------------------------------------------------------------
# description  Starts the monitor and starts the export of data via $self->SUPER::drive().
#========================================================================================
sub drive {
    my $self = shift;

    $self->startMonitor('Monitor');
    $self->SUPER::drive();
    return;
}

#========================================================================================
# function     exportLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->exportLine();
#----------------------------------------------------------------------------------------
# description  Exports one data line, get the line element (object) from function topID('Item')
#               get attribute value from object, writes line.
#               Increments the progress monitor.
#========================================================================================
sub exportLine {
    my $self = shift;

    my $Object = $self->topID('Item');
    my @AttributeAliases = map { $_->alias } @{$self->columns};
    my $hValues = $Object->get(\@AttributeAliases);
    $self->writeLine( [@{$hValues}{@AttributeAliases}]);

    $self->monitorIncrement;
    $self->monitorContent;
    return;
}

#========================================================================================
# function     startMonitor
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->startMonitor($PageTypeName);
#----------------------------------------------------------------------------------------
# description  Starts the progress monitor.
#----------------------------------------------------------------------------------------
# input        $PageTypeName | name of pagetype | string
#========================================================================================
sub startMonitor {
    my $self = shift;
    my ($PageTypeName) = @_;

    $self->{'Monitor'} = DE_EPAGES::Presentation::API::Object::ProgressMonitor->new(
        'PageTypeName' => $PageTypeName,
        'Object'       => $self->object,
    ) unless exists $self->{'Monitor'};
    return;
}

#========================================================================================
# function     monitorContent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->monitorContent;
#----------------------------------------------------------------------------------------
# description  Write the monitor page to a file.
#========================================================================================
sub monitorContent {
    my $self = shift;

    my $Monitor = $self->monitor;
    $Monitor->write if defined $Monitor;
    return;
}

#========================================================================================
# function     monitorIncrement
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->monitorIncrement;
#----------------------------------------------------------------------------------------
# description  Increments the monitor counter.
#========================================================================================
sub monitorIncrement {
    my $self = shift;

    my $Monitor = $self->monitor;
    $Monitor->incCurrent if defined $Monitor;
    return;
}
1;

