#========================================================================================
# package      DE_EPAGES::Zettle::API::WebService::InventoryService
# base         DE_EPAGES::Zettle::API::WebService::RestService
# state        private
#----------------------------------------------------------------------------------------
# description  Service wrapper around inventory-related methods (aka stock levels) from Zettle Inventory API
#               See https://github.com/iZettle/api-documentation/blob/master/inventory.adoc
#               See https://inventory.izettle.com/swagger#/
#========================================================================================
package DE_EPAGES::Zettle::API::WebService::InventoryService;
use base DE_EPAGES::Zettle::API::WebService::RestService;

use strict;

use DE_EPAGES::Zettle::API::WebService::DTO::LocationInventoryDTO;
use DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO;
use DE_EPAGES::Zettle::API::WebService::EndpointHandler qw (
    GetBulkUpdateProductTrackingApiUrl
    GetDisableProductTrackingApiUrl
    GetEnableProductTrackingApiUrl
    GetLocationTypeInventoryApiUrl
    GetLocationInventoryApiUrl
    GetProductInventoryApiUrl
    GetUpdateProductBalanceApiUrl
    GetBulkUpdateProductBalanceApiUrl
);

# Maximum number of items to send in a single bulk request
use constant BATCH_SIZE => 500;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Zettle::API::WebService::InventoryService->new($Environment);
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Environment | a Zettle environment object | DE_EPAGES::Zettle::API::Object:ZettleEnvironment
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Zettle::API::WebService::InventoryService
#========================================================================================
sub new {
    my $class = shift;
    my ($Environment) = @_;

    my $self = $class->SUPER::new($Environment);
    return $self;
}

#========================================================================================
# function     enableProductTracking
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductInventoryDTO = $self->enableProductTracking($ProductUuid);
#----------------------------------------------------------------------------------------
# description  Start inventory tracking for given product.
#               See: https://inventory.izettle.com/swagger#/inventory/startTrackingProduct
#----------------------------------------------------------------------------------------
# return       $ProductInventoryDTO | a ProductInventoryDTO object | DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO
#========================================================================================
sub enableProductTracking {
    my $self = shift;

    my ($ProductUuid) = @_;

    my $InventoryUrl = GetEnableProductTrackingApiUrl();
    my $hHeadersParams = {
        'Content-type' => 'application/json'
    };
    my $hContent = {
        'productUuid' => $ProductUuid
    };

    my $HttpResponse = $self->sendPostRequest($InventoryUrl, $hHeadersParams, $hContent);
    my $hLocationBalance = $self->decodeResponseContent($HttpResponse);

    my $ProductInventoryDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO->fromHash($hLocationBalance);

    return $ProductInventoryDTO;
}

#========================================================================================
# function     disableProductTracking
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HttpResponse = $self->disableProductTracking($ProductUuid);
#----------------------------------------------------------------------------------------
# description  Stop inventory tracking for given product.
#               See: https://inventory.izettle.com/swagger#/inventory/inactivateTrackingForProduct
#========================================================================================
sub disableProductTracking {
    my $self = shift;

    my ($ProductUuid) = @_;

    my $InventoryUrl = GetDisableProductTrackingApiUrl($ProductUuid);

    my $HttpResponse = $self->sendDeleteRequest($InventoryUrl);

    return $HttpResponse;
}

#========================================================================================
# function     bulkUpdateProductTracking
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hProductBalanceBulk = $self->bulkUpdateProductTracking($aProductUuid, $TrackingStatus, $LocationUuid);
#----------------------------------------------------------------------------------------
# description  Update inventory information for given products.
#               See: https://inventory.izettle.com/swagger#/inventory/bulkUpdate
#----------------------------------------------------------------------------------------
# input        $aProductUuid | List of product UUIDs | ref.array.string
# input        $TrackingStatus | The new tracking status to set for updated products, any of: START_TRACKING, STOP_TRACKING, NO_CHANGE | string
# input        $LocationUuid | optional. UUID of location where updated balances will be fetched from (default: STORE) | string
#----------------------------------------------------------------------------------------
# return       $hProductBalanceBulk | List of updated variation balances for location, with the following keys:
#                                      <ul>
#                                          <li>locationUuid - Identifies the location - string
#                                          <li>variants - List of balances - array.hash
#                                              <ul>
#                                                  <li>productUuid - Identifies the product - string
#                                                  <li>variantUuid - Identifies the variant - string
#                                                  <li>balance - Balance of variation in location - string
#                                              </ul>
#                                      </ul> | ref.hash.*
#========================================================================================
# throws       WebServiceError - If $TrackingStatus is STOP_TRACKING and some of the UUIDs in $aProductUuid are not present in Zettle
#               the request will fail on the first missing UUID and not update the rest of the products. The error is:
#               WebserviceError: Response={"developerMessage":"Product not tracked. UUID=61180610-1f42-1948-2520-b976395f1ee2","errorType":"PRODUCT_NOT_TRACKED","violations":[]}; Status=404; StatusCode=404 Not Found
#               To complete pending updates caller will have to invoke the method again passing the rest of the products.
#========================================================================================
sub bulkUpdateProductTracking {
    my $self = shift;

    my ($aProductUuid, $TrackingStatus, $LocationUuid) = @_;
    
    # Default location is STORE
    $LocationUuid //= $self->getEnvironment()->locationStoreUuid();

    my $InventoryUrl = GetBulkUpdateProductTrackingApiUrl();
    my $hHeadersParams = {
        'Content-type' => 'application/json'
    };
    #TODO: implement BulkInventoryUpdateRequestDTO, ProductChangeDTO and VariantChangeDTO
    
    # Make batches since call is limited to 500 items
    my $hProductBalanceBulk;
    my $ProductCount = scalar @{$aProductUuid};
    for (my $StartIndex=0; $StartIndex<$ProductCount; $StartIndex+=BATCH_SIZE){
        my $EndIndex = ($StartIndex+BATCH_SIZE >= $ProductCount) ? $ProductCount-1 : $StartIndex+BATCH_SIZE-1;
        my @aBatchProductUuid = @{$aProductUuid}[$StartIndex..$EndIndex];
        my @ahProductChange;
        foreach my $ProductUuid (@aBatchProductUuid){
            push(@ahProductChange,
                    {
                        'productUuid' => $ProductUuid,
                        'trackingStatusChange' => $TrackingStatus
                    });
        }
        my $hContent = {
            'productChanges' => \@ahProductChange,
            'returnBalanceForLocationUuid' => $LocationUuid
        };
        
        my $HttpResponse = $self->sendPostRequest($InventoryUrl, $hHeadersParams, $hContent);
        my $BatchResponse = $self->decodeResponseContent($HttpResponse);
        if(defined $hProductBalanceBulk){
            # Merge response content with data processed in previous iterations
            push(@{$hProductBalanceBulk->{'variants'}}, @{$BatchResponse->{'variants'}});
        }else{
            $hProductBalanceBulk = $BatchResponse;
        }
    }

    #TODO: implement ProductBalanceBulkDTO (check whether ProductInventoryDTO could be "reused")
    return $hProductBalanceBulk;
}

#========================================================================================
# function     getLocationTypeInventory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LocationInventoryDTO = $self->getLocationTypeInventory($LocationType);
#----------------------------------------------------------------------------------------
# description  Retrieve all product inventory information for given type of location.
#               See: https://inventory.izettle.com/swagger#/inventory/getInventoryByLocationType
#----------------------------------------------------------------------------------------
# input        $LocationType | Type of location, any of: SUPPLIER, STORE, SOLD, BIN | string
#----------------------------------------------------------------------------------------
# return       $LocationInventoryDTO | a LocationInventoryDTO object | DE_EPAGES::Zettle::API::WebService::DTO::LocationInventoryDTO
#========================================================================================
sub getLocationTypeInventory {
    my $self = shift;
    my ($LocationType) = @_;

    my $Url = GetLocationTypeInventoryApiUrl();

    my $QueryParams = {
        'type' => $LocationType
    };
    my $hOptions = {
        'content' => $QueryParams
    };
    my $HttpResponse = $self->sendGetRequest($Url, $hOptions);
    my $hLocationInventory = $self->decodeResponseContent($HttpResponse);
    my $LocationInventoryDTO = DE_EPAGES::Zettle::API::WebService::DTO::LocationInventoryDTO->fromHash($hLocationInventory);

    return $LocationInventoryDTO;
}

#========================================================================================
# function     getLocationInventory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LocationInventoryDTO = $self->getLocationInventory();
# syntax       $LocationInventoryDTO = $self->getLocationInventory($LocationUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve all product inventory information for given location.
#               If no location is provided STORE location inventory will be retrieved.
#               See: https://inventory.izettle.com/swagger#/inventory/getInventoryForLocation
#----------------------------------------------------------------------------------------
# input        $LocationUUID | optional. Location UUID (default: STORE location UUID) | string
#----------------------------------------------------------------------------------------
# return       $LocationInventoryDTO | a LocationInventoryDTO object | DE_EPAGES::Zettle::API::WebService::DTO::LocationInventoryDTO
#========================================================================================
sub getLocationInventory {
    my $self = shift;
    my ($LocationUUID) = @_;

    $LocationUUID //= $self->getEnvironment()->locationStoreUuid();

    my $Url = GetLocationInventoryApiUrl($LocationUUID);

    my $HttpResponse = $self->sendGetRequest($Url);
    my $hLocationInventory = $self->decodeResponseContent($HttpResponse);
    my $LocationInventoryDTO = DE_EPAGES::Zettle::API::WebService::DTO::LocationInventoryDTO->fromHash($hLocationInventory);

    return $LocationInventoryDTO;
}

#========================================================================================
# function     getProductInventory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductInventoryDTO = $self->getProductInventory($ProductUuid);
# syntax       $ProductInventoryDTO = $self->getProductInventory($ProductUuid, $LocationUuid);
#----------------------------------------------------------------------------------------
# description  Retrieve inventory for product in given location.
#               See: https://inventory.izettle.com/swagger#/inventory/getInventoryForProduct
#----------------------------------------------------------------------------------------
# input        $ProductUuid | Product UUID | string
# input        $LocationUUID | optional. Location UUID (default: STORE location UUID) | string
#----------------------------------------------------------------------------------------
# return       $ProductInventoryDTO | a ProductInventoryDTO object | DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO
#========================================================================================
sub getProductInventory {
    my $self = shift;
    my ($ProductUuid, $LocationUuid) = @_;

    $LocationUuid //= $self->getEnvironment()->locationStoreUuid();

    my $Url = GetProductInventoryApiUrl($LocationUuid, $ProductUuid);

    my $HttpResponse = $self->sendGetRequest($Url);
    my $hProductInventory = $self->decodeResponseContent($HttpResponse);
    my $ProductInventoryDTO = DE_EPAGES::Zettle::API::WebService::DTO::ProductInventoryDTO->fromHash($hProductInventory);

    return $ProductInventoryDTO;
}

#========================================================================================
# function     updateProductBalance
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductBalanceDTO = $self->updateProductBalance($ProductUuid, $VariantUuid, $BalanceChange);
#----------------------------------------------------------------------------------------
# description  Update balance for a single product variation.
#               See: https://inventory.izettle.com/swagger#/inventory/updateBalanceForProduct
#----------------------------------------------------------------------------------------
# input        $ProductUuid | Product UUID | string
# input        $VariantUuid | Variation UUID | string
# input        $BalanceChange | Change in product stock level | integer
#----------------------------------------------------------------------------------------
# return       $ProductBalanceDTO | a ProductBalanceDTO object | DE_EPAGES::Zettle::API::WebService::DTO::ProductBalanceDTO
#========================================================================================
sub updateProductBalance {
    my $self = shift;
    my ($ProductUuid, $VariantUuid, $BalanceChange) = @_;

    my $UpdateBalanceUrl = GetUpdateProductBalanceApiUrl();

    my $hHeadersParams = {
        'Content-type' => 'application/json'
    };
    my $hContent = $self->_getUpdateProductBalanceContent($ProductUuid, $VariantUuid, $BalanceChange);

    my $HttpResponse = $self->sendPutRequest($UpdateBalanceUrl, $hHeadersParams, $hContent);

    my $hProductBalance = $self->decodeResponseContent($HttpResponse);
    
    return $hProductBalance;
}

#========================================================================================
# function     bulkUpdateProductBalance
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductBalanceBulkDTO = $self->bulkUpdateProductBalance($hBalanceUpdate);
#----------------------------------------------------------------------------------------
# description  Update balance for multiple products.
#               See: https://inventory.izettle.com/swagger#/inventory/bulkUpdate
#----------------------------------------------------------------------------------------
# input        $hBalanceUpdate | Map of balance updates indexed by ProductUUID
#                                 where values are a map of variation stock changes indexed by VariantUUID | ref.hash.hash
#----------------------------------------------------------------------------------------
# return       $ProductBalanceBulkDTO | a ProductBalanceBulkDTO object | DE_EPAGES::Zettle::API::WebService::DTO::ProductBalanceBulkDTO
#========================================================================================
sub bulkUpdateProductBalance {
    my $self = shift;
    my ($hBalanceUpdate) = @_;

    my $BulkUpdateBalanceUrl = GetBulkUpdateProductBalanceApiUrl();

    my $hHeadersParams = {
        'Content-type' => 'application/json'
    };
    
    # Make batches since call is limited to 500 items
    my $hProductBalanceBulk;
    my @aProductUuid = keys %{$hBalanceUpdate};
    my $ProductCount = scalar @aProductUuid;
    for (my $StartIndex=0; $StartIndex<$ProductCount; $StartIndex+=BATCH_SIZE){
        my $EndIndex = ($StartIndex+BATCH_SIZE >= $ProductCount) ? $ProductCount-1 : $StartIndex+BATCH_SIZE-1;
        my @aBatchProductUuid = @aProductUuid[$StartIndex..$EndIndex];
        # Use slice of hash keys to obtain a "hash slice"
        my %hBatchBalanceUpdate = map { $_ => $hBalanceUpdate->{$_} } @aBatchProductUuid;

        my $hContent = $self->_getBulkUpdateProductBalanceContent(\%hBatchBalanceUpdate);
        
        my $HttpResponse = $self->sendPostRequest($BulkUpdateBalanceUrl, $hHeadersParams, $hContent);

        my $BatchResponse = $self->decodeResponseContent($HttpResponse);
        
        if(defined $hProductBalanceBulk){
            # Merge response content with data processed in previous iterations
            push(@{$hProductBalanceBulk->{'variants'}}, @{$BatchResponse->{'variants'}});
        }else{
            $hProductBalanceBulk = $BatchResponse;
        }
    }

    return $hProductBalanceBulk;
}


#========================================================================================


sub _getUpdateProductBalanceContent {
    my $self = shift;
    my ($ProductUuid, $VariantUuid, $BalanceChange) = @_;

    my $Environment = $self->getEnvironment();
    
    my $LocationSoldUuid = $Environment->locationSoldUuid();
    my $LocationStoreUuid = $Environment->locationStoreUuid();
    my $LocationSupplierUuid = $Environment->locationSupplierUuid();

    my ($FromLocationUuid, $ToLocationUuid);
    if($BalanceChange < 0){
        $FromLocationUuid = $LocationStoreUuid;
        $ToLocationUuid = $LocationSoldUuid;
    }else{
        $FromLocationUuid = $LocationSupplierUuid;
        $ToLocationUuid = $LocationStoreUuid;
    }

    #TODO: implement UpdateBalanceRequestDTO and ProductVariantChangeDTO
    my $AbsValue = abs($BalanceChange);
    my @ahUpdateBalance;
    push(@ahUpdateBalance,
            {
                'productUuid' => $ProductUuid,
                'variantUuid' => $VariantUuid,
                'fromLocationUuid' => $FromLocationUuid,
                'toLocationUuid' => $ToLocationUuid,
                'change' => "$AbsValue"
            });
    # externalUuid is sent back within the payload of some incoming callbacks like InventoryBalanceChanged event
    # External identifier is used to avoid processing our own stock updates sent back via incoming stock callbacks
    my $hContent = {
        'externalUuid' => $Environment->getMerchantUUID(),
        'returnBalanceForLocationUuid' => $LocationStoreUuid,
        'changes' => \@ahUpdateBalance,
    };

    return $hContent;
}

sub _getBulkUpdateProductBalanceContent {
    my $self = shift;
    my ($hBalanceUpdate) = @_;

    my $Environment = $self->getEnvironment();
    
    my $LocationSoldUuid = $Environment->locationSoldUuid();
    my $LocationStoreUuid = $Environment->locationStoreUuid();
    my $LocationSupplierUuid = $Environment->locationSupplierUuid();

    #TODO: implement BulkUpdateRequestDTO, ProductChangeDTO and VariantChangeDTO
    my @ahProductChange;
    my ($FromLocationUuid, $ToLocationUuid);

    foreach my $ProductUuid (keys %{$hBalanceUpdate}){
        my $hVariantUpdates = $hBalanceUpdate->{$ProductUuid};
        my @aVariantChanges;
        foreach my $VariantUuid (keys %{$hVariantUpdates}){
            my $BalanceChange = $hVariantUpdates->{$VariantUuid};
            if($BalanceChange < 0){
                $FromLocationUuid = $LocationStoreUuid;
                $ToLocationUuid = $LocationSoldUuid;
            }else{
                # Returned products may increase stock but we cannot tell when updates are due to returns so we always work with SUPPLIER and STORE locations
                $FromLocationUuid = $LocationSupplierUuid;
                $ToLocationUuid = $LocationStoreUuid;
            }
            push(@aVariantChanges,
                    {
                        'variantUuid' => $VariantUuid,
                        'fromLocationUuid' => $FromLocationUuid,
                        'toLocationUuid' => $ToLocationUuid,
                        'change' => abs($BalanceChange)
                    });
        }
        push(@ahProductChange,
                {
                    'productUuid' => $ProductUuid,
                    'trackingStatusChange' => 'NO_CHANGE',
                    'variantChanges' => \@aVariantChanges,
                });
    }

    my $hContent = {
        'returnBalanceForLocationUuid' => $LocationStoreUuid,
        'productChanges' => \@ahProductChange,
    };

    return $hContent;
}

1;
