use strict;

use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning BlockWarnings );
use DE_EPAGES::Core::API::Log qw (LogError );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw ( ExistsClassByAlias LoadClassByAlias LoadObjectByPath LoadObject ExistsObject);
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::Zettle::API::Constants qw ( ZETTLE_ENVIRONMENT_CLASS ZETTLE_OBJECT_ALIAS );

# Store to be used if 'storename' argument is not present
use constant DEFAULT_STORE => 'Store';

my $ShopAlias;

# Remove duplicate ZettleUUID from products
sub Main {

    local $| = 1;

    my ($Help, $NoOutput, $Password, $StoreName);

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias
    );

    $StoreName //= DEFAULT_STORE;

    usage() if $Help;

    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            # make sure that the cartridge is installed
            unless(ExistsCartridgeByPackage('DE_EPAGES::Zettle')) {
                printf STDERR ("[%s] WARN: Cartridge Zettle is not installed.\n", $StoreName) unless $NoOutput;
                return;
            }

            my @Environments = ();
            # Run script on a specific shop if 'shop' argument is present (and different from 'all')
            if(defined $ShopAlias && $ShopAlias ne 'all'){
                my $Shop;
                eval {
                    $Shop = LoadObjectByPath('/Shops')->child( $ShopAlias );
                };
                if(ExistsError()){
                    printf STDERR ("[%s] WARN: Shop '%s' does not exist.\n", $StoreName, $ShopAlias);
                    return;
                }

                return unless $Shop->existsChild(ZETTLE_OBJECT_ALIAS);

                my $ZettleEnvironment = $Shop->child(ZETTLE_OBJECT_ALIAS);

                push (@Environments, $ZettleEnvironment);
            } else {
                unless(ExistsClassByAlias(ZETTLE_ENVIRONMENT_CLASS)){
                    printf STDERR ("[%s] WARN: Class '%s' does not exist.\n", $StoreName, ZETTLE_ENVIRONMENT_CLASS);
                    return;
                }

                # Retrieve all Zettle instances from store
                my $aZettleEnvironments = LoadClassByAlias(ZETTLE_ENVIRONMENT_CLASS)->objects;
                
                foreach my $ZettleEnvironment ( @{ $aZettleEnvironments }){
                    push (@Environments, $ZettleEnvironment);
                }
            }

            RunRemoveDuplicateZettleUUIDsJob(\@Environments, $StoreName, $NoOutput);
        },
    );

    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $timemsg = sprintf( 'need %.3f seconds', $seconds );
    print $timemsg. "\n" unless $NoOutput;

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub RunRemoveDuplicateZettleUUIDsJob {
    my ($Environments, $StoreName, $NoOutput) = @_;
    
    return unless scalar @{$Environments};

    my $aWarnings = BlockWarnings(
        sub {
            my $dbi=GetCurrentDBHandle();
            my $AttributeID = LoadClassByAlias('Product')->attribute('ZettleUUID')->id;
            foreach my $Environment ( @{$Environments} ) {
                my $EnvPath = $Environment->pathString;
                my $Shop = $Environment->getSite();
                eval{
                    my $QueryProductsWithSameZettleUUID = <<'END_SQL';
select sa1.objectid, sa1.value
from stringattribute sa1
    inner join (
            select value, count(*)
            from stringattribute sa2
            where sa2.attributeid=?
            group by value
            having count(*)>1) sa2
        on sa1.value=sa2.value
    inner join object os on sa1.objectid=os.objectid
    where os.siteid=?
    order by sa1.value, sa1.objectid
END_SQL

                    my $aaRows= $dbi->execute($QueryProductsWithSameZettleUUID,[$AttributeID,$Shop->id]);

                    #Results are ordered by zeetleuuid and objectid
                    my $hOriginalProductInfo;
                    foreach my $aRow(@$aaRows){
                        my $ProductID = $aRow->[0];
                        my $ZettleUUID = $aRow->[1];

                        if(!defined($hOriginalProductInfo) || $hOriginalProductInfo->{'ZettleUUID'} ne $ZettleUUID) {
                            $hOriginalProductInfo = {
                                'ProductID' => $ProductID,
                                'ZettleUUID' => $ZettleUUID
                            };
                        } elsif($hOriginalProductInfo->{'ZettleUUID'} eq $ZettleUUID) {
                            if(ExistsObject($ProductID)){
                                my $CloneProduct = LoadObject($ProductID);
                                print sprintf("[%s] Product %d: removed ZettleUUID=%s duplicated from product id %s",$Shop->alias, $ProductID, $ZettleUUID, $hOriginalProductInfo->{ProductID})."\n";
                                $CloneProduct->set({'ZettleUUID' => undef});
                            }
                        }
                    }
                };
                if(ExistsError()){
                    my $Error = GetError();
                    $Error->addVars({ 'Path' => $EnvPath });
                    Warning($Error);
                }
            }
        }
    );

    foreach my $Warning ( @{$aWarnings} ) {
        LogError($Warning);

        printf STDERR ("[%s] WARN: %s: %s\n",
                        $StoreName,
                        $Warning->vars->{Path}//'path unknown' ,
                        substr( $Warning->shortMessage, 0, 4096 ) ); ## no critic (ProhibitMagicNumbers)
    }

    return;
}

sub usage {
    print <<END_USAGE;
Description:
    Delete duplicates ZettleUUID in products.

Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Default: Store)
    -shop         shop alias             (optional)   (Default: all)
flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -shop DemoShop
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
