#==============================================================================
# package      DE_EPAGES::Zettle::UI::WebhookHelper
# state        public
#==============================================================================
package DE_EPAGES::Zettle::UI::WebhookHelper;
use base qw( Exporter );

use strict;

our @EXPORT_OK = qw ( SetupWebhooks );

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Zettle::API::Constants qw ( %INITIAL_EXPORT_STATUS );
use DE_EPAGES::Zettle::API::ZettleService;
use DE_EPAGES::Zettle::API::ZettleUrlBuilder;


#========================================================================================
# function     SetupWebhooks
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetupWebhooks($Servlet, $Environment);
#----------------------------------------------------------------------------------------
# description  Configure external webhooks so that shop receives asynchronous updates from Zettle.
#               Multiple events are available but we only subscribe to stock related events:
#               <ul>
#                   <li> InventoryTrackingStarted
#                   <li> InventoryTrackingStopped
#                   <li> InventoryBalanceChanged
#               </ul>
#               For supported events see: https://github.com/iZettle/api-documentation/blob/master/pusher-api/api-reference.md#supported-events
#----------------------------------------------------------------------------------------
# input        $Servlet | a Servlet object | DE_EPAGES::Presentation::Servlet::BasePermission
# input        $Environment | environment object | DE_EPAGES::Zettle::API::Object::ZettleEnvironment
#========================================================================================
sub SetupWebhooks {
    my ($Servlet, $Environment) = @_;

    my $Shop = $Environment->getSite();

    my $UrlBuilder = _CreateZettleUrlBuilder();
    my $Destination = $UrlBuilder->buildWebhookStockUrl($Environment);
    # Contact email to notify webhook errors: use merchant email or fallback to shop email
    my $ContactEmail;
    if (defined($Servlet->user)) {
        my $ServletUserEmail = $Servlet->user()->get('EMail');
        $ContactEmail = $ServletUserEmail if isValid($ServletUserEmail);
    }
    $ContactEmail //= $Shop->get('Address')->get('EMail');

    my $Service = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
    $Service->setupStockWebhook($Destination, $ContactEmail);
}


#----------------------------------------------------------------------------------------
# Private methods
#----------------------------------------------------------------------------------------

sub _CreateZettleUrlBuilder {
    return DE_EPAGES::Zettle::API::ZettleUrlBuilder->new();
}

1;